// src/index.ts
import {transform} from "esbuild";
import filesPlugin from "vite-plugin-files";

// src/options.ts
function resolveOptions(userOptions) {
  const {
    routesDir = [{dir: "src/pages", baseRoute: ""}],
    extensions = ["tsx"],
    exclude = [/components/],
    async = false
  } = userOptions;
  return Object.assign({}, userOptions, {
    routesDir,
    extensions,
    exclude,
    async
  });
}

// src/generateRoutes.ts
function makePath(parentPath, filename) {
  return `${parentPath}/${filename.replace(/^\$/, ":").replace(/\$$/, "?")}`.replace(/\/\//, "/");
}
function makeFileName(name) {
  return name.substring(0, name.lastIndexOf("."));
}
function makeFilePath(path) {
  if (path[0] !== "/")
    path = "/" + path;
  return path;
}
function generatedFile(tree, parentPath) {
  const filename = makeFileName(tree.name);
  const currentPath = makePath(parentPath, filename);
  if (filename == "index") {
    console.log("currentPath", currentPath);
  }
  return {path: currentPath, file: makeFilePath(tree.relativePath)};
}
function generateRoutesRecurse(tree, parentPath, isRoot) {
  console.log("generateRoutesRecurse", tree);
  let generatedRoutes = [];
  if (!tree.children || tree.children.length <= 0)
    return generatedRoutes;
  const currentPath = makePath(parentPath, isRoot ? "/" : tree.name);
  const filesMap = {};
  const specialFilesMap = {};
  const dirsMap = {};
  tree.children.forEach((el) => {
    if (el.type === "file") {
      const filename = makeFileName(el.name);
      const isSpacialFile = /^(_layout|index|_loading|404)$/.test(filename);
      if (isSpacialFile) {
        specialFilesMap[filename] = el;
      } else {
        filesMap[el.name] = el;
      }
    }
    if (el.type === "directory")
      dirsMap[el.name] = el;
  });
  const dirRoutes = Object.keys(dirsMap).map((key) => generateRoutesRecurse(dirsMap[key], currentPath, false)).reduce((pre, next) => pre.concat(next), []);
  generatedRoutes = generatedRoutes.concat(dirRoutes);
  const fileRoutes = Object.keys(filesMap).map((key) => generatedFile(filesMap[key], currentPath));
  generatedRoutes = generatedRoutes.concat(fileRoutes);
  if (specialFilesMap["index"]) {
    const indexRoute = generatedFile(specialFilesMap["index"], currentPath);
    indexRoute.path = indexRoute.path.replace(/\/index$/, "/");
    indexRoute.exact = true;
    generatedRoutes.push(indexRoute);
  }
  if (specialFilesMap["404"]) {
    const notFoundRoute = generatedFile(specialFilesMap["404"], currentPath);
    notFoundRoute.path = `${currentPath}*`;
    notFoundRoute.exact = false;
    generatedRoutes.push(notFoundRoute);
  }
  if (specialFilesMap["_layout"]) {
    generatedRoutes = [
      {path: currentPath, file: makeFilePath(specialFilesMap["_layout"].relativePath), routes: generatedRoutes}
    ];
  }
  return generatedRoutes;
}
var generateRoutes = (trees) => {
  const routes = trees.map(([tree, pageDir]) => {
    let {baseRoute} = pageDir;
    return generateRoutesRecurse(tree, baseRoute, true);
  }).reduce((pre, next) => pre.concat(next), []);
  return routes;
};

// src/generateString.ts
function stringifyRoute(route) {
  const {path, file, exact, routes} = route;
  const routesString = routes ? `routes:[${routes.map((el) => stringifyRoute(el)).join(",\n")}]` : "";
  const exactString = `exact:${routes ? "false" : !!exact},`;
  return `
        {
            path: "${path}",
            component: loadable(() => import("${file}")),
            ${exactString}
            ${routesString}
        }
    `;
}
var stringifyRoutes = (routes) => {
  return {stringRoutes: `[${routes.map((el) => stringifyRoute(el)).join(",\n")}]`};
};
var assembleCode = (stringRoutes) => {
  console.log("stringRoutes", stringRoutes);
  return `
        import React, { PropsWithoutRef, ComponentType } from "react";
        import { BrowserRouter as Router, Switch, Route, Link } from "react-router-dom";
        import loadable from "@loadable/component";
        
        export interface RouteConfig {
            path: string;
            exact: boolean;
            component: ComponentType;
            routes?: RouteConfig[];
        }
        
        function RouteWithSubRoutes(route: PropsWithoutRef<RouteConfig>) {
            return (
                <Route
                    exact={route.exact}
                    path={route.path}
                    render={(props) => (
                        <route.component {...props}>{route.routes && renderRoute(route.routes)}</route.component>
                    )}
                />
            );
        }
        
        function renderRoute(routes: RouteConfig[]) {
            return (
                <Switch>
                    {routes.map((route, i) => (
                        <RouteWithSubRoutes key={i} {...route} />
                    ))}
                </Switch>
            );
        }

        export const routes = ${stringRoutes}
        
        export default function RouterRoot() {
            return <Router>{renderRoute(routes)}</Router>;
        }
    `;
};

// src/index.ts
function ViteReactRouter(userOptions) {
  const options = resolveOptions(userOptions);
  const {routesDir, extensions, exclude} = options;
  return filesPlugin({
    name: "vite-plugin-react-router",
    virtualId: "generated-routes",
    filesDir: routesDir,
    extensions,
    exclude,
    onGeneratedClient: (trees) => {
      const routes = generateRoutes(trees);
      const {stringRoutes} = stringifyRoutes(routes);
      const code = assembleCode(stringRoutes);
      return code;
    }
  }, {
    async transform(_code, id) {
      if (id === "/generated-routes") {
        return await transform(_code, {loader: "tsx"});
      }
    }
  });
}
var src_default = ViteReactRouter;
export {
  src_default as default
};
